#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* Tamanho mximo em caracteres dos diversos componentes da agenda */
#define TAM_COD      9
#define TAM_DESCR    21

struct prod {
  char codigo[TAM_COD];
  char descr[TAM_DESCR];
  int qtdEstoque;
  int qtdVendido;
  float preco;
};

/* define tiop PRODUTO */
typedef struct prod PRODUTO;

/* L do teclado os dados de um item */
/* Se usuario digita ENTER como codigo, o membro codigo 
   assume como valor um string vazio (valor '\0' no primeiro elemento 
   do vetor)
*/
PRODUTO leItem ()
{
  PRODUTO item;

  printf("CODIGO: ");
  fgets(item.codigo, TAM_COD, stdin);
  item.codigo[strlen(item.codigo)-1] = '\0';

  if (strlen(item.codigo)) {

    printf("DESCRICAO: ");
    fgets(item.descr, TAM_DESCR, stdin);
    item.descr[strlen(item.descr)-1] = 0;

    printf("Quantidade Estoque: ");
    scanf("%d", &item.qtdEstoque);
    getchar();

    printf("Preo unitario R$: ");
    scanf("%f", &item.preco);
    getchar();
    
    item.qtdVendido = 0;
    
  }

  return item;
}

/* ordena os itens de um inventrio pela qtde em estoque */
void ordena(PRODUTO *inventario, int nelem)
{
  int i, j;
  PRODUTO aux;

  for(i=0; i < nelem; ++i)
    for(j=i+1; j < nelem; ++j)
      if (inventario[i].qtdEstoque > inventario[j].qtdEstoque){
	aux = inventario[i];
	inventario[i] = inventario[j];
	inventario[j] = aux;
      }

}


/* Imprime na tela os valores de um PRODUTO */
void imprimeItem (PRODUTO item)
{
  printf("%9s %20s %5d %5.2f %5d \n", item.codigo, item.descr, 
	 item.qtdEstoque, item.preco, item.qtdVendido);
}

void imprimeInventario(PRODUTO *inventario, int nelem)
{
  int i = 0;

  printf("\n\nCodigo  Descr         Qtde    Preco    Vendido\n");
  printf("================================================\n\n");

  ordena(inventario, nelem);

  while (i < nelem)
    imprimeItem(inventario[i++]);

  printf("\n\n");
}

/* Verifica se j existe um item com o cdigo indicado.
   Retorna o ndice do vetor onde encontrou item.
   Se ndice retornado  igual ao nmero de elementos, 
   no existe cdigo.
*/
int existeItem(PRODUTO *inventario, int nelem, char * cod)
{
  int n = 0;
  while (n < nelem)
  {
    if (!strcmp(cod,inventario[n].codigo))
      return n;
    n++;
  }

  return n;
}


/* Grava inventario em arquivo */
int gravaInventario (PRODUTO *inventario, int nelem, FILE *fp)
{
  int status = 1;

  if (fwrite(inventario, sizeof(PRODUTO), nelem, fp) != nelem)
    status = 0;

  return status;
}

/* L de arquivo todos os itens do inventario 
   Retorna rea alocada para vetor com itens.
   Parmetro de sada nelem indica a quantidade de itens
   do inventrio.
*/
PRODUTO *leInventario (FILE *fp, int *nelem)
{
  PRODUTO *inventario;
  int i = 0;

  inventario = (PRODUTO *) calloc (1, sizeof(PRODUTO));
  
  if (inventario != NULL)
    while (fread(inventario+i, sizeof(PRODUTO), 1, fp) == 1)
    {
      inventario = (PRODUTO *) realloc (inventario, (i + 2) * sizeof(PRODUTO));
      i++;
    }

  *nelem = i;
  return inventario;
}


main (int argc, char *argv[])
{

  PRODUTO *inventario;
  char *arquivo = "inventario.dat";
  int nitens, item;
  FILE *fp;

  if (argc == 1)
  {
    /* Tenta abrir arquivo para acrescentar dados */
    fp = fopen (arquivo, "a+");
    
    /* Se houve erro, considera que arquivo no existe 
       e cria um novo arquivo (vazio)
    */
    if (fp == NULL)
	fp = fopen (arquivo, "w+");

    /* Cria vetor para novos itens do inventrio */
    inventario = (PRODUTO *) calloc (0, sizeof(PRODUTO));
    
    nitens = 0;
    do {
      inventario = (PRODUTO *) realloc (inventario,
					(nitens + 1) * sizeof(PRODUTO));
      if (inventario)
	inventario[nitens] = leItem();
      if (nitens != (item = existeItem(inventario, nitens, 
				       inventario[nitens].codigo)))
	inventario[item]=inventario[nitens--];
    } while (inventario && strlen(inventario[nitens++].codigo));
    
    gravaInventario(inventario, nitens - 1, fp);
    fclose(fp);
  }
  else if (strcmp (argv[1], "-l") == 0)
  {
    fp = fopen (arquivo, "r");
    if (fp)
    {
      if ((inventario = leInventario(fp, &nitens)) != NULL)
	imprimeInventario(inventario, nitens);
    }
    fclose(fp);
  }
  else if (strcmp (argv[1], "-q") == 0)
  {
    fp = fopen (arquivo, "r+");
    if (fp)
    {
      if ((inventario = leInventario(fp, &nitens)) != NULL)
      {
	int qtd;

	item = existeItem(inventario, nitens, argv[2]);
	if (item < nitens)
	{
	  printf ("Qtde Estoque: ");
	  scanf ("%d", &qtd);

	  if (qtd < inventario[item].qtdEstoque)
	    inventario[item].qtdVendido += inventario[item].qtdEstoque - qtd;

	  inventario[item].qtdEstoque = qtd;

	  rewind (fp);
	  gravaInventario(inventario, nitens, fp);
	}
      }
    }
    fclose(fp);
  }
  else if (strcmp (argv[1], "-v") == 0)
  {
    fp = fopen (arquivo, "r");
    if (fp)
    {
      if ((inventario = leInventario(fp, &nitens)) != NULL)
      {
	int i = 0;
	float total = 0;

	while (i < nitens)
	{
	  total = total + inventario[i].qtdVendido * inventario[i].preco;
	  i++;
	}
	printf ("Total de Vendas: R$ %5.2f\n", total);
      }
    }
    fclose(fp);
  }
  else
  {
    printf ("Opcao invlida\n");
    return(1);
  }
}
